//
//  WZAudioPlayer.h
//  AudioSDK
//
//  Copyright © 2018 Waze. All rights reserved.
//

#import <UIKit/UIKit.h>

@class WZPlayerConfiguration;
@class WZAudioPlaylist;
@class WZAudioPlaylistGroup;
@class WZAudioPlayerState;
@class WZAudioTrack;
@protocol WZAudioPlayerDelegate;

NS_ASSUME_NONNULL_BEGIN

/**
 * Singleton providing access to Waze Audio Kit integration.
 *
 * This class is *not* thread-safe.
 * Calls to instance methods should be serialised using a serial dispatch queue, such as the main
 * queue.
 */
@interface WZAudioPlayer : NSObject

/** Delegate to inform of SDK and audio player events. */
@property(nonatomic, weak) id<WZAudioPlayerDelegate> delegate;

/** Configuration for Waze's audio player and certain SDK behavior. */
@property(nonatomic, copy) WZPlayerConfiguration *configuration;

/** View to display a Waze banner in your app, that will automatically show turn-by-turn directions
 * when appropriate. */
@property(nonatomic, readonly) UIView *bannerView;

/** `YES` if the SDK is connected to the Waze app. */
@property(nonatomic, assign, readonly) BOOL isConnected;

/** Unavailable, use `sharedInstance`. */
- (instancetype)init NS_UNAVAILABLE;

/** Unavailable, use `sharedInstance`. */
- (instancetype)new;

/** Singleton for the Waze Audio Kit. */
+ (instancetype)sharedInstance;

/** The Waze app icon which appears in the banner. */
+ (UIImage *)wazeAppIcon;

/** `YES` if the device is connected to a car stereo. */
+ (BOOL)connectedToCar;

/** `YES` if Waze is installed on this device. */
+ (BOOL)isWazeInstalled;

/** Opens the App store to install Waze. */
+ (void)installWazeIfNeeded;

/**
 This method should be called from `-[UIApplicationDelegate application:openURL:options:]`

 @param url Provided from `-application:openURL:options:`
 @param options Provided from `-application:openURL:options:`
 @return `YES` if the URL was handled.
 */
- (BOOL)handleURL:(NSURL *)url options:(NSDictionary<UIApplicationOpenURLOptionsKey, id> *)options;

/** Connect to Waze Audio Kit. */
- (void)connect;

/**
 Disconnect from Waze. You may call this if your app provides a user setting for Waze integration
 and the user disables it.
 */
- (void)disconnect;

/**
 Update details for the current track, including player options.

 @param track State of the track describing its details and actions that can be performed.
 */
- (void)updateTrackState:(WZAudioTrack *)track;

/**
 Update state of playback in the audio player.

 @param state State of the player describing playback position, speed, and whether audio is being
 played.
 */
- (void)updatePlayerState:(WZAudioPlayerState *)state;

/**
 Update the contents of the playlist browser.

 @param playlistGroups Updated list of playlist groups.
 */
- (void)updatePlaylistGroups:(NSArray<WZAudioPlaylistGroup *> *)playlistGroups;

/**
 Displays an error message in Waze displaying the given message, and offers the user the option to
 return to your app.

 @param message Error message to be displayed to the user.
 */
- (void)showErrorMessageInWaze:(NSString *)message;

/**
 Update the highlighted item in the playlist browser. Setting to nil indicates the user is not
 listening to a playlist.

 @param playlist The playlist to show as currently playing.
 */
- (void)updateSelectedPlaylist:(nullable WZAudioPlaylist *)playlist;

@end

NS_ASSUME_NONNULL_END
