//
//  WZAudioPlayerDelegate.h
//  WazeSDK
//
//  Created by Eytan Biala on 6/13/18.
//  Copyright © 2018 Waze. All rights reserved.
//

#ifndef WZAudioPlayerDelegate_h
#define WZAudioPlayerDelegate_h

#import "WZPlayerConfiguration.h"

@class WZAudioPlayer;
@class WZNavigationStatus;

/**
 Receives updates from `WZAudioPlayer` whenever the user interacts with Waze's audio player.
 */
@protocol WZAudioPlayerDelegate <NSObject>

/**
 Notifies your app that the player connected to or disconnected from Waze.
 Upon a successful connection your app should generally call:
 @c [player updateTrackState:] with your current audio track,
 @c [player updatePlaybackState:] with the current playback status of your audio track,
 @c [player updatePlaylistGroups:] to show your user's playlists within Waze.

 @param player The player instance.
 @param isConnected @c YES if the player is connected to the Waze app, @c NO otherwise.
 */
- (void)wazeAudioPlayer:(WZAudioPlayer *)player didChangeConnectionStatus:(BOOL)isConnected;

/**
 The user tapped play/pause. Be sure to call @c [player updatePlayerState:] to update
 playback position and playback state.

 @param player The connected player.
 @param isPlaying @c YES to resume playback, @c NO to pause playback.
 */
- (void)wazeAudioPlayer:(WZAudioPlayer *)player requestedSetIsPlaying:(BOOL)isPlaying;

/**
 Skip to the next track. Be sure to call @c [player updateTrackState:] with the next track, and
 @c [player updatePlayerState:] to update playback position.

 @param player The connected player.
 */
- (void)wazeAudioPlayerRequestedNextTrack:(WZAudioPlayer *)player;

/**
 Skip to the previous track. You can decide whether this restarts the current track or returns to
 the previous track based on the current playback position. Be sure to call
 @c [player updateTrackState:] if the track was changed, and
 @c [player updatePlayerState:] to update playback position.

 @param player The connected player.
 */
- (void)wazeAudioPlayerRequestedPreviousTrack:(WZAudioPlayer *)player;

/**
 Update the favoriting status for the current track.
 You must call @c [player updateTrackState:] providing the current track with an
 updated @c WZAudioTrackOptions that reflects the new favorited state.

 @param player The connected player.
 @param action The favoriting action the user selected.
 */
- (void)wazeAudioPlayer:(WZAudioPlayer *)player requestedAction:(WZAudioPlayerButtonType)action;

/**
 Begin playing the specified playlist.

 @param player The connected player.
 @param playlist The playlist the user selected.
 */
- (void)wazeAudioPlayer:(WZAudioPlayer *)player requestedChangePlaylist:(WZAudioPlaylist *)playlist;

@optional

/**
 Called with navigation status changes. You may use this to implement a custom banner within your
 app to show turn-by-turn directions, or recommend content/playlists based on location
 and destination.

 @param player The connected player.
 @param navigationUpdate Current navigation status in Waze.
 */
- (void)wazeAudioPlayer:(WZAudioPlayer *)player didReceiveNavigationUpdate:(WZNavigationStatus *)navigationUpdate;

/**
 Rewind the current audio track. Call @c [player updatePlayerState:] to
 update with the new playback position.

 @param player The connected player.
 @param duration Number of seconds to rewind.
 */
- (void)wazeAudioPlayer:(WZAudioPlayer *)player requestedRewindWithDuration:(NSTimeInterval)duration;

/**
 Advance forward in the current audio track. Call @c [player updatePlayerState:] to
 update with the new playback position.

 @param player The connected player.
 @param duration Number of seconds to advance.
 */
- (void)wazeAudioPlayer:(WZAudioPlayer *)player requestedFastForwardWithDuration:(NSTimeInterval)duration;

/**
 Signals the delegate that the @c player opened the Waze application.

 @param player The connected player.
 */
- (void)wazeAudioPlayerDidOpenWaze:(WZAudioPlayer *)player;

/**
 Signals the delegate that the user wants to close the banner for the @c player.
 If implemented, this should handle removing the banner from the UI stack.

 @param player The connected player.
 */
- (void)wazeAudioPlayerRequestedCloseBanner:(WZAudioPlayer *)player;

/**
 Signals the delegate that the given @c player detected that the user connected to a car. The
 delegate should show the Waze banner.

 @param player The connected player.
 */
- (void)wazeAudioPlayerDetectedConnectionToCar:(WZAudioPlayer *)player;

/**
 Signals the delegate that the given @c player detected that the user disconnected from a car. The
 delegate should hide the Waze banner.

 @param player The connected player.
 */
- (void)wazeAudioPlayerDisconnectedFromCar:(WZAudioPlayer *)player;

@end

#endif /* WZAudioPlayerDelegate_h */
